Log("log", "background.js loaded");

var hostName = "cz.ica.icapkiservice.host";
var nativeport = null;
var pagePort = null;
var timeout = null;

var loggingEnabled = false;
var pageClosedTimeout = 10;
var inactivePageTimeout = 600;

var pageClosedTimeoutMin = 2;
var inactivePageTimeoutMin = 60;
var DIR = "directive"
var DIR_TERMINATE = "close";
var DIR_EXTENSION_CONNECTED = "connectExtension";
var DIR_EXTENSION_VERSION = "extensionVersion";
/***************** Load Options *******************************/

function LoadOptions() {
    chrome.storage.sync.get({
        loggingEnabled: false,
        pageClosedTimeout: 10,
        inactivePageTimeout: 600
    }, function(items) {
        loggingEnabled = items.loggingEnabled;

        pageClosedTimeout = items.pageClosedTimeout;
        if(pageClosedTimeout < pageClosedTimeoutMin)
            pageClosedTimeout = pageClosedTimeoutMin;

        inactivePageTimeout = items.inactivePageTimeout;
        if(inactivePageTimeout < inactivePageTimeoutMin)
            inactivePageTimeout = inactivePageTimeoutMin;
        Log("info", "LoadOptions(): loggingEnabled: " + loggingEnabled + "\npageClosedTimeout: " + pageClosedTimeout + "\ninactivePageTimeout: " + inactivePageTimeout);
    });
}
/***************** Logging ************************************/

function Log(level, message) {
    if(loggingEnabled) {
        switch (level) {
            case "log":
                console.log(message);
                break;
            case "info":
                console.info(message);
                break;
            case "warn":
                console.warn(message);
                break;
            case "error":
                console.error(message);
                break;
            default :
                console.log(message);
                break;
        }
    }
}

/************* Communication functions to page*****************/
//process message from the page
function onMessageFromPage(m, sender, responseCallback) {
    Log("log", "onMessageFromPage: Received message from: " + sender.tab + ". Message text: " + JSON.stringify(m));

    //process when directive
    if(m.type == DIR) {
        if(m.content == DIR_EXTENSION_CONNECTED) {
            //send the same response to the page
            sendMessageToPage(Directive(DIR_EXTENSION_CONNECTED));
        }
        else if(m.content == DIR_EXTENSION_VERSION) {
            sendMessageToPage(Directive({DIR_EXTENSION_VERSION: GetExtensionVersion()}));
        }
    } else {
        sendNativeMessage(m);
    }
}

function onMessagingFromPageDisconnected() {
    Log("info", "onMessagingFromPageDisconnected: Messaging from page disconnected. Setting 'port = null', closing ICAPKIServiceHost");
    pagePort = null;
    resetTimeout(pageClosedTimeout);
}

//send message to the page
function sendMessageToPage(m) {
    Log("log", "sendMessageToPage: Sending message to the page: " + JSON.stringify(m));


    if(pagePort != null) {
        pagePort.postMessage(m);
    }
    else
        Log("error", "Page port is null! Unable to send message to page.")


}

function Directive(m) {
    return {type: DIR, content: m};
}

function GetExtensionVersion() {
    var manifestData = chrome.app.getDetails();
    return manifestData.version;
}

/************* Native Messaging Communication functions *****************/
function connectNativeHost() {
    Log("log", "connectNativeHost: Trying to connect nativeHost: " + hostName);
    if(nativeport == null) {
        Log("log", "connectNativeHost: Connection 'nativeport == null', establishing connection to the native host");
        nativeport = chrome.runtime.connectNative(hostName);
        if(nativeport == null) {
            Log("error", "connectNativeHost: Unable to establish connection to the native host");
            return false;
        }
        nativeport.onMessage.addListener(onNativeMessage);
        nativeport.onDisconnect.addListener(onNativeMessagingDisconnect);
    }
    else {
        Log("log", "connectNativeHost: Connection 'nativeport != null', connection to the native host already established");
    }
    return true;
}

function onNativeMessagingDisconnect() {
    Log("info", "onNativeMessagingDisconnect: Native messaging disconnected from native host. Setting 'port = null'");
    nativeport = null;
    pagePort.disconnect();
    pageport = null;
}

function terminateNativeHost() {
    Log("info", "terminateNativeHost: Sending the close message to the ICAPKIServiceHost");
    sendNativeMessage(Directive(DIR_TERMINATE));
    nativeport.disconnect();
    nativeport = null;
    pagePort.disconnect();
    pagePort = null;
}

//send native message to the host app
function sendNativeMessage (m) {
    if(m == null) {
        Log("error", "sendNativeMessage: Message to be send is empty!");
        return;
    }
    if(!connectNativeHost()) {
        Log("error", "sendNativeMessage: No connection to the native host")
        return;
    }

    Log("log", "sendNativeMessage: Sending message to the native host: " + JSON.stringify(m));
    nativeport.postMessage(m);

    resetTimeout(inactivePageTimeout);
};

//process message from native messaging
function onNativeMessage(m) {
    Log("log", "onNativeMessage: Received message from the native host: " + JSON.stringify(m));
    sendMessageToPage(m);
}

function resetTimeout(seconds) {
    Log("log", "resetTimeout: Reseting timeout. Automatic termination of the host app in " + seconds + " seconds");
    clearTimeout(timeout);
    timeout = setTimeout(terminateNativeHost, seconds*1000);
}

/************* OnLoad commands *****************/

chrome.runtime.onConnectExternal.addListener(function(portToPage) {
    LoadOptions();
    Log("log", "onConnectExternal.addListener: Background page connected to the page through pagePort");
    pagePort = portToPage;
    pagePort.onMessage.addListener(onMessageFromPage);
    pagePort.onDisconnect.addListener(onMessagingFromPageDisconnected);
});